/*
 * Definitions of target machine for GNU compiler, for Kalray kvx.
 * Copyright (C) 2009-2019 Kalray S.A.
 *
 * This file is part of GCC.
 *
 * GCC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * GCC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GCC; see the file COPYING3.  If not see
 * <http://www.gnu.org/licenses/>.
 */

enum reg_class {
  NO_REGS,
  GPR_REGS,
  SAB_REGS,
  SFR_REGS,
  XCR_REGS,
  ALL_REGS,
  LIM_REG_CLASSES
};

#define REG_CLASS_NAMES { \
  "NO_REGS", \
  "GPR_REGS", \
  "SAB_REGS", \
  "SFR_REGS", \
  "XCR_REGS", \
  "ALL_REGS", \
  "LIM_REG_CLASSES" \
}

#define KVX_GPR_FIRST_REGNO 0
#define KVX_GPR_LAST_REGNO 63
#define KVX_SFR_FIRST_REGNO 64
#define KVX_SFR_LAST_REGNO 255
#define KVX_XCR_FIRST_REGNO 256
#define KVX_XCR_LAST_REGNO 511

#define KVX_MDS_REGISTERS 512
#define FIRST_PSEUDO_REGISTER (KVX_MDS_REGISTERS)
#define KVX_FRAME_POINTER_VIRT_REGNO (KVX_SFR_FIRST_REGNO + 102)

#define KVX_SFR_FAKE_GPR_COUNT 16
#define KVX_SFR_FAKE_GPR_MASK (KVX_SFR_FAKE_GPR_COUNT - 1)
#define KVX_SFR_FAKE_GPR_FIRST (KVX_SFR_LAST_REGNO - (KVX_SFR_FAKE_GPR_MASK))

#define REG_CLASS_CONTENTS { \
  { 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, }, /* NO_REGS */ \
  { 0xffffffff, 0xffffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xffff0000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, }, /* GPR_REGS */ \
  { 0x00000000, 0x00000000, 0x10000002, 0x00000000, 0x000000f0, 0x0000003f, 0x00000010, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, }, /* SAB_REGS */ \
  { 0x00000000, 0x00000000, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x0000ffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, }, /* SFR_REGS */ \
  { 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, }, /* XCR_REGS */ \
  { 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, }, /* ALL_REGS */ \
}

#define REGNO_REG_CLASS(REGNO) ( \
  (((REGNO) <= KVX_GPR_LAST_REGNO)? GPR_REGS: \
    (((REGNO) <= KVX_SFR_LAST_REGNO-KVX_SFR_FAKE_GPR_COUNT)? SFR_REGS: \
      (((REGNO) <= KVX_SFR_LAST_REGNO)? GPR_REGS: \
        (((REGNO) <= KVX_XCR_LAST_REGNO)? XCR_REGS: \
          NO_REGS)))))

#define KVX_REGISTER_NAMES \
	"r0",	"r1",	"r2",	"r3",	"r4",	"r5",	"r6",	"r7", \
	"r8",	"r9",	"r10",	"r11",	"r12",	"r13",	"r14",	"r15", \
	"r16",	"r17",	"r18",	"r19",	"r20",	"r21",	"r22",	"r23", \
	"r24",	"r25",	"r26",	"r27",	"r28",	"r29",	"r30",	"r31", \
	"r32",	"r33",	"r34",	"r35",	"r36",	"r37",	"r38",	"r39", \
	"r40",	"r41",	"r42",	"r43",	"r44",	"r45",	"r46",	"r47", \
	"r48",	"r49",	"r50",	"r51",	"r52",	"r53",	"r54",	"r55", \
	"r56",	"r57",	"r58",	"r59",	"r60",	"r61",	"r62",	"r63", \
	"pc",	"ps",	"pcr",	"ra",	"cs",	"csit",	"aespc",	"ls", \
	"le",	"lc",	"ipe",	"men",	"pmc",	"pm0",	"pm1",	"pm2", \
	"pm3",	"pmsa",	"tcr",	"t0v",	"t1v",	"t0r",	"t1r",	"wdv", \
	"wdr",	"ile",	"ill",	"ilr",	"mmc",	"tel",	"teh",	"ixc", \
	"syo",	"hto",	"ito",	"do",	"mo",	"pso",	"tpcm0",	"tpcm1", \
	"dc",	"dba0",	"dba1",	"dwa0",	"dwa1",	"mes",	"ws",	"dc0", \
	"dc1",	"dc2",	"dc3",	"dba2",	"dba3",	"dwa2",	"dwa3",	"tpcm2", \
	"tpcmc",	"pm4",	"pm5",	"pm6",	"pm7",	"pmc2",	"srhpc",	"frcc", \
	"spc_pl0",	"spc_pl1",	"spc_pl2",	"spc_pl3",	"sps_pl0",	"sps_pl1",	"sps_pl2",	"sps_pl3", \
	"ea_pl0",	"ea_pl1",	"ea_pl2",	"ea_pl3",	"ev_pl0",	"ev_pl1",	"ev_pl2",	"ev_pl3", \
	"sr_pl0",	"sr_pl1",	"sr_pl2",	"sr_pl3",	"es_pl0",	"es_pl1",	"es_pl2",	"es_pl3", \
	"sid_pl0",	"sid_pl1",	"sid_pl2",	"sid_pl3",	"sr1_pl0",	"sr1_pl1",	"sr1_pl2",	"sr1_pl3", \
	"syow",	"htow",	"itow",	"dow",	"mow",	"psow",	"res102",	"res103", \
	"tpcc_pl0",	"tpcc_pl1",	"tpcc_pl2",	"tpcc_pl3",	"res108",	"res109",	"res110",	"res111", \
	"res112",	"res113",	"res114",	"res115",	"res116",	"res117",	"res118",	"res119", \
	"res120",	"res121",	"res122",	"res123",	"res124",	"res125",	"res126",	"res127", \
	"spc",	"res129",	"res130",	"res131",	"sps",	"res133",	"res134",	"res135", \
	"ea",	"res137",	"res138",	"res139",	"ev",	"res141",	"res142",	"res143", \
	"sr",	"res145",	"res146",	"res147",	"es",	"res149",	"res150",	"res151", \
	"sid",	"res153",	"res154",	"res155",	"sr1",	"res157",	"res158",	"res159", \
	"res160",	"res161",	"res162",	"res163",	"res164",	"res165",	"res166",	"res167", \
	"tpcc",	"res169",	"res170",	"res171",	"res172",	"res173",	"res174",	"res175", \
	"res176",	"res177",	"res178",	"res179",	"res180",	"res181",	"res182",	"res183", \
	"res184",	"res185",	"res186",	"res187",	"res188",	"res189",	"res190",	"res191", \
	"a0_x",	"a0_y",	"a0_z",	"a0_t",	"a1_x",	"a1_y",	"a1_z",	"a1_t", \
	"a2_x",	"a2_y",	"a2_z",	"a2_t",	"a3_x",	"a3_y",	"a3_z",	"a3_t", \
	"a4_x",	"a4_y",	"a4_z",	"a4_t",	"a5_x",	"a5_y",	"a5_z",	"a5_t", \
	"a6_x",	"a6_y",	"a6_z",	"a6_t",	"a7_x",	"a7_y",	"a7_z",	"a7_t", \
	"a8_x",	"a8_y",	"a8_z",	"a8_t",	"a9_x",	"a9_y",	"a9_z",	"a9_t", \
	"a10_x",	"a10_y",	"a10_z",	"a10_t",	"a11_x",	"a11_y",	"a11_z",	"a11_t", \
	"a12_x",	"a12_y",	"a12_z",	"a12_t",	"a13_x",	"a13_y",	"a13_z",	"a13_t", \
	"a14_x",	"a14_y",	"a14_z",	"a14_t",	"a15_x",	"a15_y",	"a15_z",	"a15_t", \
	"a16_x",	"a16_y",	"a16_z",	"a16_t",	"a17_x",	"a17_y",	"a17_z",	"a17_t", \
	"a18_x",	"a18_y",	"a18_z",	"a18_t",	"a19_x",	"a19_y",	"a19_z",	"a19_t", \
	"a20_x",	"a20_y",	"a20_z",	"a20_t",	"a21_x",	"a21_y",	"a21_z",	"a21_t", \
	"a22_x",	"a22_y",	"a22_z",	"a22_t",	"a23_x",	"a23_y",	"a23_z",	"a23_t", \
	"a24_x",	"a24_y",	"a24_z",	"a24_t",	"a25_x",	"a25_y",	"a25_z",	"a25_t", \
	"a26_x",	"a26_y",	"a26_z",	"a26_t",	"a27_x",	"a27_y",	"a27_z",	"a27_t", \
	"a28_x",	"a28_y",	"a28_z",	"a28_t",	"a29_x",	"a29_y",	"a29_z",	"a29_t", \
	"a30_x",	"a30_y",	"a30_z",	"a30_t",	"a31_x",	"a31_y",	"a31_z",	"a31_t", \
	"a32_x",	"a32_y",	"a32_z",	"a32_t",	"a33_x",	"a33_y",	"a33_z",	"a33_t", \
	"a34_x",	"a34_y",	"a34_z",	"a34_t",	"a35_x",	"a35_y",	"a35_z",	"a35_t", \
	"a36_x",	"a36_y",	"a36_z",	"a36_t",	"a37_x",	"a37_y",	"a37_z",	"a37_t", \
	"a38_x",	"a38_y",	"a38_z",	"a38_t",	"a39_x",	"a39_y",	"a39_z",	"a39_t", \
	"a40_x",	"a40_y",	"a40_z",	"a40_t",	"a41_x",	"a41_y",	"a41_z",	"a41_t", \
	"a42_x",	"a42_y",	"a42_z",	"a42_t",	"a43_x",	"a43_y",	"a43_z",	"a43_t", \
	"a44_x",	"a44_y",	"a44_z",	"a44_t",	"a45_x",	"a45_y",	"a45_z",	"a45_t", \
	"a46_x",	"a46_y",	"a46_z",	"a46_t",	"a47_x",	"a47_y",	"a47_z",	"a47_t", \
	"a48_x",	"a48_y",	"a48_z",	"a48_t",	"a49_x",	"a49_y",	"a49_z",	"a49_t", \
	"a50_x",	"a50_y",	"a50_z",	"a50_t",	"a51_x",	"a51_y",	"a51_z",	"a51_t", \
	"a52_x",	"a52_y",	"a52_z",	"a52_t",	"a53_x",	"a53_y",	"a53_z",	"a53_t", \
	"a54_x",	"a54_y",	"a54_z",	"a54_t",	"a55_x",	"a55_y",	"a55_z",	"a55_t", \
	"a56_x",	"a56_y",	"a56_z",	"a56_t",	"a57_x",	"a57_y",	"a57_z",	"a57_t", \
	"a58_x",	"a58_y",	"a58_z",	"a58_t",	"a59_x",	"a59_y",	"a59_z",	"a59_t", \
	"a60_x",	"a60_y",	"a60_z",	"a60_t",	"a61_x",	"a61_y",	"a61_z",	"a61_t", \
	"a62_x",	"a62_y",	"a62_z",	"a62_t",	"a63_x",	"a63_y",	"a63_z",	"a63_t",

#define KVX_PGR_REGISTER_NAMES \
	"r0r1",	"r2r3",	"r4r5",	"r6r7",	"r8r9",	"r10r11",	"r12r13",	"r14r15", \
	"r16r17",	"r18r19",	"r20r21",	"r22r23",	"r24r25",	"r26r27",	"r28r29",	"r30r31", \
	"r32r33",	"r34r35",	"r36r37",	"r38r39",	"r40r41",	"r42r43",	"r44r45",	"r46r47", \
	"r48r49",	"r50r51",	"r52r53",	"r54r55",	"r56r57",	"r58r59",	"r60r61",	"r62r63",

#define KVX_QGR_REGISTER_NAMES \
	"r0r1r2r3",	"r4r5r6r7",	"r8r9r10r11",	"r12r13r14r15",	"r16r17r18r19",	"r20r21r22r23",	"r24r25r26r27",	"r28r29r30r31", \
	"r32r33r34r35",	"r36r37r38r39",	"r40r41r42r43",	"r44r45r46r47",	"r48r49r50r51",	"r52r53r54r55",	"r56r57r58r59",	"r60r61r62r63",

#define KVX_XBR_REGISTER_NAMES \
	"a0_lo",	"a0_hi",	"a1_lo",	"a1_hi",	"a2_lo",	"a2_hi",	"a3_lo",	"a3_hi", \
	"a4_lo",	"a4_hi",	"a5_lo",	"a5_hi",	"a6_lo",	"a6_hi",	"a7_lo",	"a7_hi", \
	"a8_lo",	"a8_hi",	"a9_lo",	"a9_hi",	"a10_lo",	"a10_hi",	"a11_lo",	"a11_hi", \
	"a12_lo",	"a12_hi",	"a13_lo",	"a13_hi",	"a14_lo",	"a14_hi",	"a15_lo",	"a15_hi", \
	"a16_lo",	"a16_hi",	"a17_lo",	"a17_hi",	"a18_lo",	"a18_hi",	"a19_lo",	"a19_hi", \
	"a20_lo",	"a20_hi",	"a21_lo",	"a21_hi",	"a22_lo",	"a22_hi",	"a23_lo",	"a23_hi", \
	"a24_lo",	"a24_hi",	"a25_lo",	"a25_hi",	"a26_lo",	"a26_hi",	"a27_lo",	"a27_hi", \
	"a28_lo",	"a28_hi",	"a29_lo",	"a29_hi",	"a30_lo",	"a30_hi",	"a31_lo",	"a31_hi", \
	"a32_lo",	"a32_hi",	"a33_lo",	"a33_hi",	"a34_lo",	"a34_hi",	"a35_lo",	"a35_hi", \
	"a36_lo",	"a36_hi",	"a37_lo",	"a37_hi",	"a38_lo",	"a38_hi",	"a39_lo",	"a39_hi", \
	"a40_lo",	"a40_hi",	"a41_lo",	"a41_hi",	"a42_lo",	"a42_hi",	"a43_lo",	"a43_hi", \
	"a44_lo",	"a44_hi",	"a45_lo",	"a45_hi",	"a46_lo",	"a46_hi",	"a47_lo",	"a47_hi", \
	"a48_lo",	"a48_hi",	"a49_lo",	"a49_hi",	"a50_lo",	"a50_hi",	"a51_lo",	"a51_hi", \
	"a52_lo",	"a52_hi",	"a53_lo",	"a53_hi",	"a54_lo",	"a54_hi",	"a55_lo",	"a55_hi", \
	"a56_lo",	"a56_hi",	"a57_lo",	"a57_hi",	"a58_lo",	"a58_hi",	"a59_lo",	"a59_hi", \
	"a60_lo",	"a60_hi",	"a61_lo",	"a61_hi",	"a62_lo",	"a62_hi",	"a63_lo",	"a63_hi",

#define KVX_XVR_REGISTER_NAMES \
	"a0",	"a1",	"a2",	"a3",	"a4",	"a5",	"a6",	"a7", \
	"a8",	"a9",	"a10",	"a11",	"a12",	"a13",	"a14",	"a15", \
	"a16",	"a17",	"a18",	"a19",	"a20",	"a21",	"a22",	"a23", \
	"a24",	"a25",	"a26",	"a27",	"a28",	"a29",	"a30",	"a31", \
	"a32",	"a33",	"a34",	"a35",	"a36",	"a37",	"a38",	"a39", \
	"a40",	"a41",	"a42",	"a43",	"a44",	"a45",	"a46",	"a47", \
	"a48",	"a49",	"a50",	"a51",	"a52",	"a53",	"a54",	"a55", \
	"a56",	"a57",	"a58",	"a59",	"a60",	"a61",	"a62",	"a63",

#define KVX_XTR_REGISTER_NAMES \
	"a0a1",	"a2a3",	"a4a5",	"a6a7",	"a8a9",	"a10a11",	"a12a13",	"a14a15", \
	"a16a17",	"a18a19",	"a20a21",	"a22a23",	"a24a25",	"a26a27",	"a28a29",	"a30a31", \
	"a32a33",	"a34a35",	"a36a37",	"a38a39",	"a40a41",	"a42a43",	"a44a45",	"a46a47", \
	"a48a49",	"a50a51",	"a52a53",	"a54a55",	"a56a57",	"a58a59",	"a60a61",	"a62a63",

#define KVX_XMR_REGISTER_NAMES \
	"a0a1a2a3",	"a4a5a6a7",	"a8a9a10a11",	"a12a13a14a15",	"a16a17a18a19",	"a20a21a22a23",	"a24a25a26a27",	"a28a29a30a31", \
	"a32a33a34a35",	"a36a37a38a39",	"a40a41a42a43",	"a44a45a46a47",	"a48a49a50a51",	"a52a53a54a55",	"a56a57a58a59",	"a60a61a62a63",

#define KVX_ABI_REGULAR 0
#define KVX_PROGRAM_POINTER_REGNO 64
#define KVX_STACK_POINTER_REGNO 12
#define KVX_FRAME_POINTER_REGNO 14
#define KVX_LOCAL_POINTER_REGNO 13
#define KVX_STRUCT_POINTER_REGNO 15
#define KVX_RETURN_POINTER_REGNO 67
#define KVX_ARGUMENT_POINTER_REGNO 0
#define KVX_ARG_REG_SLOTS 12


#define KVX_ABI_REGULAR_FIXED_REGISTERS \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	1,	1,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0,

#define KVX_ABI_REGULAR_CALL_USED_REGISTERS \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	0,	1, \
	1,	1,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0,

#define KVX_ABI_REGULAR_CALL_REALLY_USED_REGISTERS \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	0,	1, \
	1,	1,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	0,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	1,	1,	1,	1,	1,	1,	1,	1, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0, \
	0,	0,	0,	0,	0,	0,	0,	0,

